package gs.test.v0.definition

import cats.data.Kleisli
import natchez.Span

trait UnitOfWork[F[_]]:

  def work(
    span: Span[F]
  ): F[Either[TestFailure, Any]]

object UnitOfWork:

  /** Instantiate a new [[UnitOfWork]] with the given function that requires a
    * `Span[F]` as input.
    *
    * @param uow
    *   The unit of work implementation.
    * @return
    *   The new [[UnitOfWork]] instance.
    */
  def apply[F[_]](
    uow: Kleisli[F, Span[F], Either[TestFailure, Any]]
  ): UnitOfWork[F] = new UnitOfWork[F] {

    override def work(span: Span[F]): F[Either[TestFailure, Any]] =
      uow.apply(span)

  }

end UnitOfWork
